/*----------------------------------------------------------------------------\
 |                       Cross Browser Tree Widget 1.17                        |
 |-----------------------------------------------------------------------------|
 |                          Created by Emil A Eklund                           |
 |                  (http://webfx.eae.net/contact.html#emil)                   |
 |                      For WebFX (http://webfx.eae.net/)                      |
 |-----------------------------------------------------------------------------|
 | An object based tree widget,  emulating the one found in microsoft windows, |
 | with persistence using cookies. Works in IE 5+, Mozilla and konqueror 3.    |
 |-----------------------------------------------------------------------------|
 |                   Copyright (c) 1999 - 2002 Emil A Eklund                   |
 |-----------------------------------------------------------------------------|
 | This software is provided "as is", without warranty of any kind, express or |
 | implied, including  but not limited  to the warranties of  merchantability, |
 | fitness for a particular purpose and noninfringement. In no event shall the |
 | authors or  copyright  holders be  liable for any claim,  damages or  other |
 | liability, whether  in an  action of  contract, tort  or otherwise, arising |
 | from,  out of  or in  connection with  the software or  the  use  or  other |
 | dealings in the software.                                                   |
 | - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
 | This  software is  available under the  three different licenses  mentioned |
 | below.  To use this software you must chose, and qualify, for one of those. |
 | - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
 | The WebFX Non-Commercial License          http://webfx.eae.net/license.html |
 | Permits  anyone the right to use the  software in a  non-commercial context |
 | free of charge.                                                             |
 | - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
 | The WebFX Commercial license           http://webfx.eae.net/commercial.html |
 | Permits the  license holder the right to use  the software in a  commercial |
 | context. Such license must be specifically obtained, however it's valid for |
 | any number of  implementations of the licensed software.                    |
 | - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
 | GPL - The GNU General Public License    http://www.gnu.org/licenses/gpl.txt |
 | Permits anyone the right to use and modify the software without limitations |
 | as long as proper  credits are given  and the original  and modified source |
 | code are included. Requires  that the final product, software derivate from |
 | the original  source or any  software  utilizing a GPL  component, such  as |
 | this, is also licensed under the GPL license.                               |
 |-----------------------------------------------------------------------------|
 | Dependencies: xtree.css (To set up the CSS of the tree classes)             |
 |-----------------------------------------------------------------------------|
 | 2001-01-10 | Original Version Posted.                                       |
 | 2001-03-18 | Added getSelected and get/setBehavior  that can make it behave |
 |            | more like windows explorer, check usage for more information.  |
 | 2001-09-23 | Version 1.1 - New features included  keyboard  navigation (ie) |
 |            | and the ability  to add and  remove nodes dynamically and some |
 |            | other small tweaks and fixes.                                  |
 | 2002-01-27 | Version 1.11 - Bug fixes and improved mozilla support.         |
 | 2002-06-11 | Version 1.12 - Fixed a bug that prevented the indentation line |
 |            | from  updating correctly  under some  circumstances.  This bug |
 |            | happened when removing the last item in a subtree and items in |
 |            | siblings to the remove subtree where not correctly updated.    |
 | 2002-06-13 | Fixed a few minor bugs cased by the 1.12 bug-fix.              |
 | 2002-08-20 | Added usePersistence flag to allow disable of cookies.         |
 | 2002-10-23 | (1.14) Fixed a plus icon issue                                 |
 | 2002-10-29 | (1.15) Last changes broke more than they fixed. This version   |
 |            | is based on 1.13 and fixes the bugs 1.14 fixed withou breaking |
 |            | lots of other things.                                          |
 | 2003-02-15 | The  selected node can now be made visible even when  the tree |
 |            | control  loses focus.  It uses a new class  declaration in the |
 |            | css file '.webfx-tree-item a.selected-inactive', by default it |
 |            | puts a light-gray rectangle around the selected node.          |
 | 2003-03-16 | Adding target support after lots of lobbying...
 2008-10-08 | Many updates for TrackStudio integration
 |-----------------------------------------------------------------------------|
 | Created 2000-12-11 | All changes are in the log above. | Updated 2008-10-08 |
 \----------------------------------------------------------------------------*/

var webFXTreeConfig = {
    rootIcon            : '',
    fileIcon            : '',
    iIcon               : '',
    lIcon               : '',
    lMinusIcon          : '',
    lPlusIcon           : '',
    tIcon               : '',
    tMinusIcon          : '',
    tPlusIcon           : '',
    blankIcon           : '',
    defaultText         : 'Tree Item',
    defaultAction       : 'javascript:void(0);',
    defaultBehavior     : 'classic',
    defaultHint         : '',
    usePersistence	    : false,
    todayNodeLocaleName : 'Today',
    FOLDER_IMG : '/'+urlHtml+'/xtree/images/folder.png'
};

var webFXTreeHandler = {
    idCounter : 0,
    idPrefix  : "webfx-tree-object-",
    all       : {},
    selected  : null,
    onSelect  : null, /* should be part of tree, not handler */
    getId     : function() { return this.idPrefix + this.idCounter++; },
    toggle    : function (oItem) { this.all[oItem.id.replace('-plus','')].toggle(); },
    select    : function (oItem) { this.all[oItem.id.replace('-icon','')].select(); },
    focus     : function (oItem) { this.all[oItem.id.replace('-anchor','')].focus(); },
    blur      : function (oItem) { this.all[oItem.id.replace('-anchor','')].blur(); },
    reload    : function (oItem) {reload(this.all[oItem.id.replace('-anchor','')].parentNode); },
    reload2   : function (oItem) {
        var tree = this.all[oItem.id.replace('-anchor','')].parentNode;
        while(tree.childNodes.length >1) {
            tree.childNodes[1].remove();
        }
        tree.loaded=false;
        tree.collapse();
        tree.expand();
    },
    keydown   : function (oItem, e) { return this.all[oItem.id].keydown(e.keyCode); },
    cookies   : new WebFXCookie(),
    insertHTMLAfterBegin : function (oElement, sHTML) {
        if (oElement != null && oElement.insertAdjacentHTML != null) {
            oElement.insertAdjacentHTML("afterBegin", sHTML);
            return;
        }
        var r = oElement.ownerDocument.createRange();
        r.setStartBefore(oElement);
        var parsedHTML = r.createContextualFragment(sHTML);
        oElement.insertBefore(parsedHTML, oElement.firstChild);
    }
}

/*
 * WebFXCookie class
 */
function WebFXCookie() {
    if (document.cookie.length) { this.cookies = ' ' + document.cookie; }
}

WebFXCookie.prototype.setCookie = function (key, value) {
    document.cookie = key + "=" + escape(value);
}

WebFXCookie.prototype.getCookie = function (key) {
    if (this.cookies) {
        var start = this.cookies.indexOf(' ' + key + '=');
        if (start == -1) { return null; }
        var end = this.cookies.indexOf(";", start);
        if (end == -1) { end = this.cookies.length; }
        end -= start;
        var cookie = this.cookies.substr(start,end);
        return unescape(cookie.substr(cookie.indexOf('=') + 1, cookie.length - cookie.indexOf('=') + 1));
    }
    else { return null; }
}

/*
 * WebFXTreeAbstractNode class
 */

function WebFXTreeAbstractNode(name, hint, action) {
    this.childNodes  = [];
    this.id     = webFXTreeHandler.getId();
    this.text   = name || webFXTreeConfig.defaultText;
    this.hint   = hint || webFXTreeConfig.defaultHint;
    this.action = action || webFXTreeConfig.defaultAction;
    this._last  = false;
    webFXTreeHandler.all[this.id] = this;
}

/**
 *       .
 */
WebFXTreeAbstractNode.prototype.add = function (node, bNoIdent) {
    node.parentNode = this;
    //console.log("this.childNodes.length", node._last);
    if (this.childNodes.length == 0) {
        node._last = true;
    }
    else {
        node._last = false;
    }
    //console.log("node._last", node._last);
    this.childNodes.unshift( node );
    var root = this;
    while (root.parentNode) { root = root.parentNode; }
    if (root.rendered) {
        //this._last = true;
        var foo = this;
        while (foo.parentNode) {
            for (var i = 0; i < foo.parentNode.childNodes.length; i++) {
                if (foo.id == foo.parentNode.childNodes[i].id) { break; }
            }
            if (i == foo.parentNode.childNodes.length - 1) { foo.parentNode._last = true; }
            else { foo.parentNode._last = false; }
            foo = foo.parentNode;
            //console.log("foo._last", foo._last);
        }
        webFXTreeHandler.insertHTMLAfterBegin(document.getElementById(this.id + '-cont'), node.toString(0, node.parentNode.childNodes.length));
        if (!this.folder) this.folder = true;
        if (!bNoIdent) this.indent();
    }

    return node;
}

WebFXTreeAbstractNode.prototype.toggle = function() {
    if (this.folder) {
        if (this.open) {
            this.collapse();
        } else {
            this.expand();
        }
    }
}

WebFXTreeAbstractNode.prototype.select = function() {
    var tempNode = document.getElementById(this.id + '-anchor');
    try{
        if (tempNode != null) tempNode.focus();
    }catch (e){}
}

WebFXTreeAbstractNode.prototype.deSelect = function() {
    if (document.getElementById(this.id + '-anchor')) {
        var className = document.getElementById(this.id + '-anchor').className;
        if (className == 'copy_selected') document.getElementById(this.id + '-anchor').className = 'copy';
        if (className == 'copy_recursively_selected') document.getElementById(this.id + '-anchor').className = 'copy_recursively';
        if (className == 'cut_selected') document.getElementById(this.id + '-anchor').className = 'cut';
        if (className != 'copy_selected' && className != 'copy_recursively_selected' && className != 'cut_selected') document.getElementById(this.id + '-anchor').className = 'deselect';
        webFXTreeHandler.selected = null;
    }
}

WebFXTreeAbstractNode.prototype.focus = function() {
    if ((webFXTreeHandler.selected) && (webFXTreeHandler.selected != this)) { webFXTreeHandler.selected.deSelect(); }
    webFXTreeHandler.selected = this;
    if (!document.getElementById(this.id)) this.id = this.getIdNode(this.hint);
    if (this.icon) { document.getElementById(this.id + '-icon').src = this.icon; }
    var className = document.getElementById(this.id + '-anchor').className;
    if (className == 'copy') document.getElementById(this.id + '-anchor').className = 'copy_selected';
    if (className == 'copy_recursively') document.getElementById(this.id + '-anchor').className = 'copy_recursively_selected';
    if (className == 'cut') document.getElementById(this.id + '-anchor').className = 'cut_selected';
    if (className != 'copy' && className != 'copy_recursively' && className != 'cut') document.getElementById(this.id + '-anchor').className = 'selected';
    //document.getElementById(this.id + '-anchor').focus();
    if (webFXTreeHandler.onSelect) { webFXTreeHandler.onSelect(this); }
}


WebFXTreeAbstractNode.prototype.blur = function() {
    if (this.icon && document.getElementById(this.id + '-icon') != null) { document.getElementById(this.id + '-icon').src = this.icon; }
}

WebFXTreeAbstractNode.prototype.doExpand = function() {
    this.id = document.getElementById(this.id) ? this.id : this.getIdNode(this.hint);
    document.getElementById(this.id + '-icon').src = this.icon;
    if (this.childNodes.length) { document.getElementById(this.id + '-cont').style.display = 'block'; }
    this.open = true;
    if (webFXTreeConfig.usePersistence) {
        webFXTreeHandler.cookies.setCookie(this.id.substr(18,this.id.length - 18), '1');
    }	}

WebFXTreeAbstractNode.prototype.doCollapse = function() {
    if (document.getElementById(this.id) != null) {
        document.getElementById(this.id + '-icon').src = this.icon;
        if (this.childNodes.length) {
            document.getElementById(this.id + '-cont').style.display = 'none';
        }
    }
    this.open = false;
    document.close();
    if (webFXTreeConfig.usePersistence) {
        webFXTreeHandler.cookies.setCookie(this.id.substr(18,this.id.length - 18), '0');
    }	}

WebFXTreeAbstractNode.prototype.expandAll = function() {
    this.expandChildren();
    if ((this.folder) && (!this.open)) { this.expand(); }
}

WebFXTreeAbstractNode.prototype.expandChildren = function() {
    for (var i = 0; i < this.childNodes.length; i++) {
        this.childNodes[i].expandAll();
    } }

WebFXTreeAbstractNode.prototype.collapseAll = function() {
    this.collapseChildren();
    if ((this.folder) && (this.open)) { this.collapse(); }
}

WebFXTreeAbstractNode.prototype.collapseChildren = function() {
    for (var i = 0; i < this.childNodes.length; i++) {
        this.childNodes[i].collapseAll();
    } }

WebFXTreeAbstractNode.prototype.indent = function(lvl, del, last, level, nodesLeft) {
    /*
     * Since we only want to modify items one level below ourself,
     * and since the rightmost indentation position is occupied by
     * the plus icon we set this to -2
     */
    if (lvl == null) { lvl = -2; }
    var state = 0;
    for (var i = this.childNodes.length - 1; i >= 0 ; i--) {
        state = this.childNodes[i].indent(lvl + 1, del, last, level);
        if (state) { return; }
    }
    if (del) {
        if ((level >= this._level) && (document.getElementById(this.id + '-plus') != null)) {
            if (this.folder) {
                document.getElementById(this.id + '-plus').src = (this.open)?webFXTreeConfig.lMinusIcon:webFXTreeConfig.lPlusIcon;
                this.plusIcon = webFXTreeConfig.lPlusIcon;
                this.minusIcon = webFXTreeConfig.lMinusIcon;
            }
            else if (nodesLeft) { document.getElementById(this.id + '-plus').src = webFXTreeConfig.tIcon; }
            return 1;
        }	}
    var foo = document.getElementById(this.id + '-indent-' + lvl);
    if (foo) {
        if ((foo._last) || ((del) && (last))) { foo.src =  webFXTreeConfig.blankIcon; }
        else { foo.src =  webFXTreeConfig.iIcon; }
    }
    return 0;
}

/*
 * WebFXTree class
 */

function WebFXTree(name, hint, action, sBehavior, icon, statusIcon) {
    this.base = WebFXTreeAbstractNode;
    this.base(name, hint, action);
    this.icon      = icon || webFXTreeConfig.rootIcon;
    this.statusIcon  = statusIcon;

    /* Defaults to open */
    if (webFXTreeConfig.usePersistence) {
        this.open  = (webFXTreeHandler.cookies.getCookie(this.id.substr(18, this.id.length - 18)) != '0');
    } else {
        this.open  = true;
    }
    this.folder    = true;
    this.rendered  = false;
    this.onSelect  = null;
    this.currentSelection = null;
}

WebFXTree.prototype = new WebFXTreeAbstractNode;

WebFXTree.prototype.getSelected = function() {
    if (webFXTreeHandler.selected) { return webFXTreeHandler.selected; }
    else { return null; }
}

WebFXTree.prototype.expand = function() {
    this.doExpand();
}

WebFXTree.prototype.getNodeByHint = function(hint) {
    for (var key in webFXTreeHandler.all) {
        var tempNode = webFXTreeHandler.all[key];
        if (tempNode != null && tempNode.hint == hint) {
            return tempNode;
        }
    }
    return null;
}

WebFXTree.prototype.renameNode = function(hint, newName) {
    var tempNode = this.getNodeByHint(hint);
    if (tempNode != null) {
        tempNode.text = newName;
        this.updateNode(hint, newName, tempNode.statusIcon);
    }
}

WebFXTree.prototype.renameNodeHint = function(hint, newHint) {
    var tempNode = this.getNodeByHint(hint);
    if (tempNode != null) {
        tempNode.hint = newHint;
        document.getElementById(tempNode.id + '-anchor').title = newHint;
    }
}

WebFXTree.prototype.updateNode = function(hint, newName, newStatus) {
    var tempNode = this.getNodeByHint(hint);
    if (tempNode != null) {
        if (newName != "") tempNode.text = newName;
        if (newStatus != "") tempNode.statusIcon = newStatus;
        //    "<"  ">"  "&lt;"  "&gt;"
        var label = tempNode.text.replace(/</g, '&lt;').replace(/>/g, '&gt;');
        if (!document.getElementById(tempNode.id)) tempNode.id = this.getIdNode(tempNode.hint);
        var _dddd = document.getElementById(tempNode.id + "-anchor") ? document.getElementById(tempNode.id + "-anchor") : document.getElementById(this.getIdNode(tempNode.hint) + "-anchor");
        var anchorInnerHTML = "";
        if (_dddd!=null) { anchorInnerHTML = _dddd.innerHTML;
            var img = anchorInnerHTML.substring(0, anchorInnerHTML.indexOf('>') + 1);
            document.getElementById(tempNode.id + "-anchor").innerHTML = img + tempNode.statusIcon + label;
        }
    }
}

WebFXTree.prototype.openGroupNode = function(hint) {
    var tempNode = this.getNodeByHint(hint);
    if (tempNode != null) {
        var _dddd = document.getElementById(tempNode.id+"-plus") ? document.getElementById(tempNode.id+"-plus") :   document.getElementById(this.getIdNode(tempNode.hint)+"-plus");
        if (_dddd!=null) {
            if (_dddd.src.indexOf('plus') != -1)
                return true;
            else
                return false;
        }
    }
}

WebFXTree.prototype.updateNodeAction = function(hint, action, status) {
    var tempNode = this.getNodeByHint(hint);
    if (tempNode != null) {
        var _dddd = document.getElementById(tempNode.id) ? document.getElementById(tempNode.id) : document.getElementById(this.getIdNode(tempNode.hint));
        var anchorInnerHTML = "";
        if (_dddd!=null) {
            anchorInnerHTML = _dddd.innerHTML;
            document.getElementById(tempNode.id).innerHTML = this.parseActionUpdateChildrensTask(anchorInnerHTML, action, status);
            anchorInnerHTML = document.getElementById(tempNode.id).innerHTML;
            var label = tempNode.text.replace(/</g, '&lt;').replace(/>/g, '&gt;');
            _dddd = document.getElementById(tempNode.id + "-anchor") ? document.getElementById(tempNode.id + "-anchor") :  document.getElementById(this.getIdNode(tempNode.hint) + "-anchor");
            if (_dddd!=null) {
                var innerHTML = _dddd.innerHTML;
                var img = innerHTML.substring(0, innerHTML.indexOf('>') + 1);
                if (this.parseActionForChildrensTaskNotClose(anchorInnerHTML) > 0)
                    document.getElementById(tempNode.id + "-anchor").innerHTML = img + tempNode.statusIcon + this.getNameParse(label, this.parseActionForChildrensTaskNotClose(anchorInnerHTML));
                else
                    document.getElementById(tempNode.id + "-anchor").innerHTML = img + tempNode.statusIcon + this.getNameParse(label, 0);
            }
        }
    }
}

WebFXTree.prototype.nameNodeAdd = function(hint) {
    var tempNode = this.getNodeByHint(hint);
    var name;
    if (tempNode != null) {
        var count = this.getCountNodeNotClose(tempNode);
        if (count>0)
            if ((tempNode.text).indexOf(" (") != -1)
                name = (tempNode.text).substring(0, (tempNode.text).indexOf(" (")) + " ("+count+")";
            else
                name = tempNode.text+ " ("+count+")";
        else
            name = tempNode.text;
    }
    return name;
}

WebFXTree.prototype.getNameParse = function(name, count) {
    if (count != 0) {
        if (name.indexOf(" (") != -1)
            name = name.substring(0, name.indexOf(" (")) + " ("+count+")";
        else
            name = name + " ("+count+")";
    }
    return name;
};

WebFXTree.prototype.actionNodeAdd = function(hint) {
    var tempNode = this.getNodeByHint(hint);
    var action;
    if (tempNode != null) {
        var _dddd = document.getElementById(tempNode.id) ? document.getElementById(tempNode.id) : null;
        if (_dddd!=null) {
            var innerHTML = _dddd.innerHTML;
            action = innerHTML.substring(0, innerHTML.lastIndexOf("';}")+4);
            action = action.substring(action.lastIndexOf("javascript"),action.length);
        }
    }
    return action;
}

WebFXTree.prototype.getCountNodeNotClose = function(node) {
    var count;
    var _dddd = document.getElementById(node.id) ? document.getElementById(node.id) : document.getElementById(this.getIdNode(node.hint));
    if (_dddd!=null) {
        var innerHTML = _dddd.innerHTML;
        count = this.parseActionForChildrensTaskNotClose(innerHTML);
    }
    return count;
}


WebFXTree.prototype.getCountNodeAll = function(node) {
    var count;
    var _dddd = document.getElementById(node.id) ? document.getElementById(node.id) : document.getElementById(this.getIdNode(node.hint));
    if (_dddd!=null) {
        var innerHTML = _dddd.innerHTML;
        count = this.parseActionForChildrensTask(innerHTML);
    }
    return count;
}

/**
 *     true -  . false -   
 * @param node 
 */
WebFXTree.prototype.isSortOrder = function(node) {
    var count;
    var _dddd = document.getElementById(node.id) ? document.getElementById(node.id) : document.getElementById(this.getIdNode(node.hint));
    if (_dddd!=null) {
        var innerHTML = _dddd.innerHTML;
        count = this.parseActionForSortOrder(innerHTML);
    }
    return count;
};

WebFXTree.prototype.getHide = function(action) {
    if (action.indexOf("hide='") != -1) {
        var hide = action.substring(action.indexOf("hide='") + "hide='".length, action.length);
        hide = hide.substring(0, hide.indexOf("'"));
        return hide;
    }
    return "F";
};

WebFXTree.prototype.setClassName = function(hint, action) {
    var tempNode = self.top.frames[0].taskTree.getNodeByHint(hint);
    if (tempNode != null)
        if (self.top.frames[0].document.getElementById(tempNode.id + '-anchor'))
            self.top.frames[0].document.getElementById(tempNode.id + '-anchor').className = action;
        else {
            var parentNode = tempNode.parentNode;
            var countChildren = parentNode.childNodes.length;
            for (var i=0;i!=countChildren;i++) {
                var candidat = parentNode.childNodes[i];
                if (tempNode.hint == candidat.hint && document.getElementById(tempNode.id))
                    document.getElementById(tempNode.id + '-anchor').className = action;
            }
        }
}

WebFXTree.prototype.parseActionForChildrensTask = function(taskAction) {
    var first = taskAction.indexOf("childs='");
    taskAction = taskAction.substring(first+8, taskAction.length);
    var end = (taskAction.substring(0, taskAction.length)).indexOf("(");
    return parseInt(taskAction.substring(0,end));
}

/**
 *  action   
 * @param taskAction action note
 */
WebFXTree.prototype.parseActionForSortOrder = function(taskAction) {
    var first = taskAction.indexOf("sortorder='") + "sortorder='".length;
    taskAction = taskAction.substring(first, taskAction.length);
    var end = (taskAction.substring(0, taskAction.length)).indexOf("'");
    return taskAction.substring(0,end);
}

WebFXTree.prototype.parseActionForChildrensTaskNotClose = function(taskAction) {
    var first = taskAction.indexOf("childs='");
    taskAction = taskAction.substring(first+8, taskAction.toString().length);
    taskAction = taskAction.substring(0, taskAction.length);
    var first = (taskAction.substring(0, taskAction.length)).indexOf("(");
    var end = (taskAction.substring(0, taskAction.length)).indexOf(")");
    return parseInt(taskAction.substring(first+1,end));
}

WebFXTree.prototype.parseActionUpdateChildrensTask = function(anchorInnerHTML, action, status) {
    var countChild = this.parseActionForChildrensTask(anchorInnerHTML);
    var countChildNotClose = this.parseActionForChildrensTaskNotClose(anchorInnerHTML);
    if (action == '+') {
        countChild++;
        if (status.indexOf('finishstate') == -1) countChildNotClose++;
    }
    if (action == '-'){
        if (status.indexOf('finishstate') == -1) countChild--;
        if (countChildNotClose > 0)  countChildNotClose--;
    }
    var firstChilds = anchorInnerHTML.indexOf("childs='");
    var tempStr = anchorInnerHTML.substring(firstChilds, anchorInnerHTML.length);
    var endChildsNotClose = tempStr.indexOf(")");
    return anchorInnerHTML.substring(0,firstChilds+8)+countChild+"("+countChildNotClose+")"+anchorInnerHTML.substring(firstChilds+endChildsNotClose+1,anchorInnerHTML.toString().length);
}

WebFXTree.prototype.getNextSibling = function() {
    return null;
}

WebFXTree.prototype.getPreviousSibling = function() {
    return null;
}

WebFXTree.prototype.keydown = function(key) {
    if (key == 39) {
        if (!this.open) { this.expand(); }
        else if (this.childNodes.length) { this.childNodes[0].select(); }
        return false;
    }
    if (key == 37) { this.collapse(); return false; }
    if ((key == 40) && (this.open) && (this.childNodes.length)) { this.childNodes[0].select(); return false; }
    return true;
}

/**
 * !!!
 *      ,
 *        ,
 * :
 * WebFXTree.prototype.updateNode = function(hint, newName, newStatus),
 * WebFXTreeItem.prototype.toString = function (nItem, nItemCount),
 * WebFXTree.prototype.renameNodeHint = function(hint, newHint),
 * WebFXTree.prototype.renameNode = function(hint, newName)
 *  . .
 */
WebFXTree.prototype.toString = function() {
    if (!this.hint) this.hint="";
    if (!this.statusIcon) this.statusIcon = "";
    var str = "<div id=\"" + this.id + "\" ondblclick=\"webFXTreeHandler.toggle(this);\" class=\"webfx-tree-item\" onkeydown=\"return webFXTreeHandler.keydown(this, event)\">" +
            "<a title=\""+ this.hint + "\" href=\"javascript:{}\" style=\"font-size: 12px;\" id=\"" + this.id + "-anchor\" onclick=\"" + this.action +"\" onfocus=\"webFXTreeHandler.focus(this);\" onblur=\"webFXTreeHandler.blur(this);\"" +
            (this.target ? " target=\"" + this.target + "\"" : "") +
            "><img id=\"" + this.id + "-icon\" style=\"margin-right: 5px;\" class=\"webfx-tree-icon\" src=\"" + this.icon + "\" onclick=\"webFXTreeHandler.select(this);\">" +
            this.statusIcon + this.text + "<\/a><\/div>" +
            "<div id=\"" + this.id + "-cont\" class=\"webfx-tree-container\" style=\"display: " + ((this.open)?'block':'none') + ";\">";
    var sb = [];

    for (var i = 0; i < this.childNodes.length; i++)
        sb[i] = this.childNodes[i].toString(i, this.childNodes.length);

    this.rendered = true;
    return str + sb.join("") + "<\/div>";
}

/**
 * WebFXTreeItem class
 * @param name
 * @param hint
 * @param action
 * @param parent
 * @param  icon
 * @param statusIcon
 */
function WebFXTreeItem(name, hint, action, parent, icon, statusIcon, noFolder) {
    this.base = WebFXTreeAbstractNode;
    this.base(name, hint, action, noFolder);
    /* Defaults to close */
    if (webFXTreeConfig.usePersistence) {
        this.open = (webFXTreeHandler.cookies.getCookie(this.id.substr(18, this.id.length - 18)) == '1');
    } else { this.open = false; }
    if (icon) { this.icon = icon; }
    if (statusIcon) { this.statusIcon = statusIcon; }
    if (parent) { parent.add(this); }
}

WebFXTreeItem.prototype = new WebFXTreeAbstractNode;

WebFXTreeItem.prototype.remove = function() {

    var iconSrc = document.getElementById(this.id + '-plus') !=null ? document.getElementById(this.id + '-plus').src : "";
    var parentNode = this.parentNode;
    var prevSibling = this.getPreviousSibling(true);
    var nextSibling = this.getNextSibling(true);
    var last = (!((nextSibling) && (nextSibling.parentNode) && (nextSibling.parentNode.id == parentNode.id)));
    this._remove();
    if (parentNode.childNodes.length == 0) {
        if (document.getElementById(parentNode.id + '-cont') != null)
            document.getElementById(parentNode.id + '-cont').style.display = 'none';
        parentNode.doCollapse();
        parentNode.folder = false;
        parentNode.open = false;
    }
    if (!nextSibling || last) { parentNode.indent(null, true, last, this._level, parentNode.childNodes.length); }
    if (document.getElementById(prevSibling.id + '-plus') && (prevSibling == parentNode) && !(parentNode.childNodes.length)) {
        prevSibling.folder = false;
        prevSibling.open = false;
        iconSrc = document.getElementById(prevSibling.id + '-plus').src;
        iconSrc = iconSrc.replace('minus', '').replace('plus', '');
        document.getElementById(prevSibling.id + '-plus').src = iconSrc;
        document.getElementById(prevSibling.id + '-icon').src = prevSibling.icon;
    }
    if (document.getElementById(prevSibling.id + '-plus')) {
        if (parentNode == prevSibling.parentNode) {
            iconSrc = iconSrc.replace('minus', '').replace('plus', '');
            document.getElementById(prevSibling.id + '-plus').src = iconSrc;
        }
    }
}

WebFXTreeItem.prototype._remove = function() {
    var i;
    for (i = this.childNodes.length - 1; i >= 0; i--) {
        if (!this.childNodes[i])
            this.childNodes[i]._remove();
    }
    for (i = 0; i < this.parentNode.childNodes.length; i++) {
        if (this == this.parentNode.childNodes[i]) {
            for (var j = i; j < this.parentNode.childNodes.length; j++) {
                this.parentNode.childNodes[j] = this.parentNode.childNodes[j+1];
            }
            this.parentNode.childNodes.length -= 1;
            if (i + 1 == this.parentNode.childNodes.length) { this.parentNode._last = true; }
            break;
        }	}
    webFXTreeHandler.all[this.id] = null;
    var tmp = document.getElementById(this.id);
    if (tmp) { tmp.parentNode.removeChild(tmp); }
    tmp = document.getElementById(this.id + '-cont');
    if (tmp) { tmp.parentNode.removeChild(tmp); }
}

WebFXTreeItem.prototype.expand = function() {
    this.doExpand();
    document.getElementById(this.id + '-plus').src = this.minusIcon;
}

WebFXTreeItem.prototype.collapse = function() {
    this.doCollapse();
    document.getElementById(this.id + '-plus').src = this.plusIcon;
}

WebFXTreeItem.prototype.getFirst = function() {
    return this.childNodes[0];
}

WebFXTreeItem.prototype.getLast = function() {
    if (this.childNodes[this.childNodes.length - 1].open) { return this.childNodes[this.childNodes.length - 1].getLast(); }
    else { return this.childNodes[this.childNodes.length - 1]; }
}

WebFXTreeItem.prototype.getNextSibling = function() {
    for (var i = 0; i < this.parentNode.childNodes.length; i++) {
        if (this == this.parentNode.childNodes[i]) { break; }
    }
    if (++i == this.parentNode.childNodes.length) { return this.parentNode.getNextSibling(); }
    else { return this.parentNode.childNodes[i]; }
}

WebFXTreeItem.prototype.getPreviousSibling = function(b) {
    for (var i = 0; i < this.parentNode.childNodes.length; i++) {
        if (this == this.parentNode.childNodes[i]) { break; }
    }
    if (i == 0) { return this.parentNode; }
    else {
        if ((this.parentNode.childNodes[--i].open) || (b && this.parentNode.childNodes[i].folder)) { return this.parentNode.childNodes[i].getLast(); }
        else { return this.parentNode.childNodes[i]; }
    }
}

WebFXTreeItem.prototype.keydown = function(key) {
    if ((key == 39) && (this.folder)) {
        if (!this.open) { this.expand(); }
        else { this.getFirst().select(); }
        return false;
    }
    else if (key == 37) {
        if (this.open) { this.collapse(); }
        else { this.parentNode.select(); }
        return false;
    }
    else if (key == 40) {
        if (this.open) { this.getFirst().select(); }
        else {
            var sib = this.getNextSibling();
            if (sib) { sib.select(); }
        }
        return false;
    }
    else if (key == 38) { this.getPreviousSibling().select(); return false; }
    return true;
}

/**
 * !!!
 *      ,
 *        ,
 * :
 * WebFXTree.prototype.updateNode = function(hint, newName, newStatus),
 * WebFXTree.prototype.toString = function(),
 * WebFXTree.prototype.renameNode = function(hint, newName),
 * WebFXTree.prototype.renameNodeHint = function(hint, newHint)
 *  . .
 */
WebFXTreeItem.prototype.toString = function (nItem, nItemCount) {
    //console.log("WebFXTreeItem.prototype.toString");
    var foo = this.parentNode;
    var indent = '';
    if (nItem + 1 == nItemCount) { foo._last = true; }
    var j = 0;
    while (foo.parentNode) {
        foo = foo.parentNode;
        indent = "<img id=\"" + this.id + "-indent-" + i + "\" src=\"" + ((foo._last)?webFXTreeConfig.blankIcon:webFXTreeConfig.iIcon) + "\">" + indent;
        j++;
    }
    this._level = i;
    if (this.childNodes.length) { this.folder = 1; }
    else { this.open = false; };

    var label = this.text.replace(/</g, '&lt;').replace(/>/g, '&gt;');

    if(!this.hint) this.hint = "";

    var pointer_cursor = "";
    if (this.folder) pointer_cursor = " class=\"pointer_cursor\"";
    if (!this.statusIcon) this.statusIcon = "";
    //console.log("this._last",this._last);
    var str = "<div id=\"" + this.id + "\" ondblclick=\"webFXTreeHandler.toggle(this);\" class=\"webfx-tree-item\" onkeydown=\"return webFXTreeHandler.keydown(this, event)\">" +
            indent +
            "<img id=\"" + this.id + "-plus\"" + pointer_cursor + " src=\"" + getConnectorImg(this.folder, this.open, this._last) + "\" onclick=\"webFXTreeHandler.toggle(this);\">" +
            "<a title=\""+ this.hint + "\" href=\"javascript:{}\" style=\"font-size: 12px;\" id=\"" + this.id + "-anchor\" onfocus=\"webFXTreeHandler.focus(this);\"" + (this.action=='javascript:{}' || this.action=='javascript:{1;}' ? 'onclick=\"webFXTreeHandler.reload' + (this.action=='javascript:{1;}' ? '2' : '') + '(this);\"' : 'onclick=\"' + this.action +'\"') + " onblur=\"webFXTreeHandler.blur(this);\"" +
            (this.target ? " target=\"" + this.target + "\"" : "") +
            ">" + "<img id=\"" + this.id + "-icon\" style=\"margin-right: 5px;\" class=\"webfx-tree-icon\" src=\"" + this.icon + "\" onclick=\"webFXTreeHandler.select(this);\">" +
            this.statusIcon + label + "<\/a><\/div>" +
            "<div id=\"" + this.id + "-cont\" class=\"webfx-tree-container\" style=\"display: " + ((this.open)?'block':'none') + ";\">";
    //console.log(str);
    var sb = [];
    for (var i = 0; i < this.childNodes.length; i++) {
        sb[i] = this.childNodes[i].toString(i, this.childNodes.length);
    }
    this.plusIcon = ((this.parentNode._last)?webFXTreeConfig.lPlusIcon:webFXTreeConfig.tPlusIcon);
    this.minusIcon = ((this.parentNode._last)?webFXTreeConfig.lMinusIcon:webFXTreeConfig.tMinusIcon);
    return str + sb.join("") + "<\/div>";
}

/*
 *        
 */
function getConnectorImg(isFolder, isOpen, isLast) {
    var connectorImg = "";
    if (isFolder) {
        if (isOpen) {
            if (isLast) {
                connectorImg = webFXTreeConfig.lMinusIcon;
            } else {
                connectorImg = webFXTreeConfig.tMinusIcon;
            }
        } else {
            if (isLast) {
                connectorImg = webFXTreeConfig.lPlusIcon;
            } else {
                connectorImg = webFXTreeConfig.tPlusIcon;
            }
        }
    } else {
        if (isLast) {
            connectorImg = webFXTreeConfig.lIcon;
        } else {
            connectorImg = webFXTreeConfig.tIcon;
        }
    }
    return connectorImg;
}

function reload(node) {
    // if loading do nothing
    if (node.loaded) {
        var open = node.open;
        // remove
        while (node.childNodes.length > 0)
            node.childNodes[node.childNodes.length - 1].remove();

        node.loaded = false;

        node._loadingItem = new WebFXTreeItem(webFXTreeConfig.loadingText);
        node.add(node._loadingItem);

        if (open)
            node.expand();
    }
    else if (node.open && !node.loading)
        _startLoadXmlTree(node.src, node);
}

WebFXTreeAbstractNode.prototype.getIdNode = function(hint) {
    var id = null;
    var tempNode = self.top.frames[0].taskTree.getNodeByHint(hint);
    var parentNode = tempNode.parentNode;
    if (parentNode.hint.indexOf("#1 ") != -1) {
        var generalNode =  parentNode.parentNode;
        var count = generalNode.childNodes.length;
        for (var i=0;i!=count;i++){
            var candidat = generalNode.childNodes[i];
            if (candidat.hint == parentNode.hint) {
                var p = self.top.frames[0].taskTree.getNodeByHint(parentNode.hint);
                parentNode.id = p.id;
                var c = p.childNodes.length;
                for (var j=0;j!=c;j++) {
                    var t = p.childNodes[j];
                    if (t.hint==tempNode.hint)
                        id = t.id;
                }
            }
        }
        tempNode.id = id;
    }
    return id;
};